package pl.mimuw.wfnetreduction.algorithms;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang3.tuple.Triple;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;

public class ContractionLog {
	ArrayList<String> stepsStrings = new ArrayList<>();
	ArrayList<Petrinet> stepsPetrinets = new ArrayList<>();
	ArrayList<Set<PetrinetNode>> stepsContractedNodes = new ArrayList<>();
	StringBuilder sb = new StringBuilder();
	Petrinet pn = null;
	Set<PetrinetNode> contrNodes = new HashSet<>();
	boolean result;
	
	public ContractionLog(Petrinet pn) {
		this.pn = PetrinetFactory.clonePetrinet(pn);
	}
	
	public void nextStep() {
		stepsStrings.add(sb.toString());
		sb = new StringBuilder();

		stepsPetrinets.add(pn);
		pn = null;
		
		stepsContractedNodes.add(contrNodes);
		contrNodes = new HashSet<>();
	}
	
	public void nextStep(Petrinet pn) {
		stepsStrings.add(sb.toString());
		sb = new StringBuilder();
		
		stepsPetrinets.add(this.pn);
		this.pn = PetrinetFactory.clonePetrinet(pn);
		
		stepsContractedNodes.add(contrNodes);
		contrNodes = new HashSet<>();		
	}
	
	public void addContractedNode(PetrinetNode n) {
		String l = n.getLabel();
		for (PetrinetNode nn : pn.getNodes()) {
			if (nn.getLabel().equals(l))
				contrNodes.add(nn);
		}
	}
	
	public void addContractedNodes(Set<PetrinetNode> nodes) {
		for (PetrinetNode n : nodes) {
			addContractedNode(n);
		}
	}

	public void println(String s) {
		sb.append(s+"\n");
	}
	
	public void println(Triple<Petrinet, Set<PetrinetNode>, Set<PetrinetNode>> triple) {
		sb.append(PetrinetHelper.toString(triple)+"\n");
	}
	
	public boolean getResult() {
		return result;
	}

	public void setResult(boolean result) {
		this.result = result;
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();
		for (String s : stepsStrings) {
			sb.append(s);
			sb.append("---\n");
		}
		return sb.toString();
	}
	
	public ArrayList<Petrinet> getPetrinets() {
		return stepsPetrinets;
	}

	public ArrayList<String> getStrings() {
		return stepsStrings;
	}

	public ArrayList<Set<PetrinetNode>> getContractedNodes() {
		return stepsContractedNodes;
	}
	
	public int getInitialSize() {
		return getPetrinets().get(0).getNodes().size();
	}
	
	public int getResultSize() {
		return getPetrinets().get(getPetrinets().size()-1).getNodes().size();
	}
	
	public void storeResultNetToFile(String fileName) throws IOException {
		//java.io.File netFile = new java.io.File(fileName);
		Petrinet pn = getPetrinets().get(getPetrinets().size()-1);
	    PrintWriter writer = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));
	    for (Place p : pn.getPlaces()) {
	    	writer.println("place \""+p.getLabel()+"\";");
	    }
	    writer.println();
	    for (Transition t : pn.getTransitions()) {
	    	writer.println("trans \""+t.getLabel()+"\"");
	    	writer.print("   in");
	    	for (PetrinetNode p : PetrinetHelper.getInPlaces(pn, t)) {
		    	writer.print(" \""+p.getLabel()+"\"");	    		
	    	}
	    	writer.println();
	    	writer.print("   out");
	    	for (PetrinetNode p : PetrinetHelper.getOutPlaces(pn, t)) {
		    	writer.print(" \""+p.getLabel()+"\"");	    		
	    	}
	    	writer.println();
	    	writer.println(";");
	    	writer.println();
	    }
	    
	    writer.close();
	}
}
