package pl.mimuw.wfnetreduction.plugins;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;

import pl.mimuw.wfnetreduction.algorithms.AndOrReductionAlgorithm;
import pl.mimuw.wfnetreduction.algorithms.ContractionLog;
import pl.mimuw.wfnetreduction.connections.WFNetReductionConnection;
import pl.mimuw.wfnetreduction.help.WFReductionHelp;
import pl.mimuw.wfnetreduction.parameters.WFNetReductionParameters;

@Plugin(name = "AND-OR reducer", parameterLabels = { "A multi I/O WF net (represented as a Petri net)",
		"Input places specification (represented as a marking of 1 meaning input place, 2 meaning output place and 3 meaning input and output place at the same time)" }, returnLabels = {
				"Contraction log (textual description of contractions and subsequent Petri nets)" }, returnTypes = { ContractionLog.class }, help = WFReductionHelp.TEXT)
public class AndOrReductionPlugin extends AndOrReductionAlgorithm {

	//	/**
	//	 * The plug-in variant that runs in any context and requires parameters.
	//	 * 
	//	 * @param context The context to run in.
	//	 * @param input1 The first input.
	//	 * @param input2 The second input.
	//	 * @param parameters The parameters to use.
	//	 * @return The output.
	//	 */
	//	@UITopiaVariant(affiliation = "University of Warsaw", author = "Your name", email = "Your e-mail address")
	//	@PluginVariant(variantLabel = "Your plug-in name, parameters", requiredParameterLabels = { 0, 1, 2 })
	//	public YourOutput run(PluginContext context, YourFirstInput input1, YourSecondInput input2, YourParameters parameters) {
	//		// Apply the algorithm depending on whether a connection already exists.
	//	    return runConnections(context, input1, input2, parameters);
	//	}

	Set<PetrinetNode> inputs = new HashSet<>();
	Set<PetrinetNode> outputs = new HashSet<>();

	static public void getIOFromMarking(Marking marking, Set<PetrinetNode> inputs, Set<PetrinetNode> outputs) {
		Set<Place> base = marking.baseSet();
		for (Place p : base) {
			if (marking.occurrences(p) == 1)
				inputs.add(p);
			else if (marking.occurrences(p) == 2)
				outputs.add(p);
			else if (marking.occurrences(p) == 3) {
				inputs.add(p);
				outputs.add(p);
			}
		}
	}

	/**
	 * The plug-in variant that runs in any context and uses the default parameters.
	 * 
	 * @param context
	 *            The context to run in.
	 * @param net
	 *            The Petri net.
	 * @param marking
	 *            The marking.
	 * @return The output.
	 */
	@UITopiaVariant(affiliation = "University of Warsaw", author = "Jacek Sroka", email = "sroka@mimuw.edu.pl")
	@PluginVariant(variantLabel = "Your plug-in name, parameters", requiredParameterLabels = { 0, 1 })
	public ContractionLog runDefault(PluginContext context, Petrinet net, Marking marking) {
		//extract input and output from marking and put into attributes of this object
		getIOFromMarking(marking, inputs, outputs);
		// Get the default parameters.
		WFNetReductionParameters parameters = new WFNetReductionParameters(net, inputs, outputs);
		// Apply the algorithm depending on whether a connection already exists.
		return runConnections(context, net, inputs, outputs, parameters);
	}

	//	/**
	//	 * The plug-in variant that runs in a UI context and uses a dialog to get the parameters.
	//	 * 
	//	 * @param context The context to run in.
	//	 * @param input1 The first input.
	//	 * @param input2 The second input.
	//	 * @return The output.
	//	 */
	//	@UITopiaVariant(affiliation = "Your affiliation", author = "Your name", email = "Your e-mail address")
	//	@PluginVariant(variantLabel = "Your plug-in name, dialog", requiredParameterLabels = { 0, 1 })
	//	public YourOutput runUI(UIPluginContext context, YourFirstInput input1, YourSecondInput input2) {
	//		// Get the default parameters.
	//	    YourParameters parameters = new YourParameters(input1, input2);
	//	    // Get a dialog for this parameters.
	//	    YourDialog dialog = new YourDialog(context, input1, input2, parameters);
	//	    // Show the dialog. User can now change the parameters.
	//	    InteractionResult result = context.showWizard("Your dialog title", true, true, dialog);
	//	    // User has close the dialog.
	//	    if (result == InteractionResult.FINISHED) {
	//			// Apply the algorithm depending on whether a connection already exists.
	//	    	return runConnections(context, input1, input2, parameters);
	//	    }
	//	    // Dialog got canceled.
	//	    return null;
	//	}	

	//	/**
	//	 * The plug-in variant that allows one to test the dialog to get the parameters.
	//	 * 
	//	 * @param context The context to run in.
	//	 * @return The output.
	//	 */
	//	@UITopiaVariant(affiliation = "Your affiliation", author = "Your name", email = "Your e-mail address")
	//	@PluginVariant(variantLabel = "Your plug-in name, dialog", requiredParameterLabels = { })
	//	public YourOutput testUI(UIPluginContext context) {
	//		// Create default inputs.
	//		YourFirstInput input1 = new YourFirstInput();
	//		YourSecondInput input2 = new YourSecondInput();
	//		// Get the default parameters.
	//	    YourParameters parameters = new YourParameters(input1, input2);
	//	    // Get a dialog for this parameters.
	//	    YourDialog dialog = new YourDialog(context, input1, input2, parameters);
	//	    // Show the dialog. User can now change the parameters.
	//	    InteractionResult result = context.showWizard("Your dialog title", true, true, dialog);
	//	    // User has close the dialog.
	//	    if (result == InteractionResult.FINISHED) {
	//			// Apply the algorithm depending on whether a connection already exists.
	//	    	return runConnections(context, input1, input2, parameters);
	//	    }
	//	    // Dialog got canceled.
	//	    return null;
	//	}	

	/**
	 * Apply the algorithm depending on whether a connection already exists.
	 * 
	 * @param context
	 *            The context to run in.
	 * @param net
	 *            Petri net
	 * @param intputs
	 *            Set of input places
	 * @param outputs
	 *            Set of output places
	 * @return The results of the reduction procedure which is a ContractionLog.
	 */
	private ContractionLog runConnections(PluginContext context, Petrinet net, Set<PetrinetNode> inputs, Set<PetrinetNode> outputs,
			WFNetReductionParameters parameters) {
		if (parameters.isTryConnections()) {
			// Try to found a connection that matches the inputs and the parameters.
			Collection<WFNetReductionConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(WFNetReductionConnection.class, context,
						net, inputs, outputs, parameters);
				for (WFNetReductionConnection connection : connections) {
					if (connection.getObjectWithRole(WFNetReductionConnection.PETRI_NET).equals(net)
							&& connection.getObjectWithRole(WFNetReductionConnection.INPUT_PLACEST).equals(inputs)
							&& connection.getObjectWithRole(WFNetReductionConnection.OUTPUT_PLACEST).equals(outputs)
							&& connection.getParameters().equals(parameters)) {
						// Found a match. Return the associated output as result of the algorithm.
						return connection.getObjectWithRole(WFNetReductionConnection.REDUCTION_RESULT);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		// No connection found. Apply the algorithm to compute a fresh output result.
		ContractionLog results = apply(context, net, inputs, outputs, parameters);
		if (parameters.isTryConnections()) {
			// Store a connection containing the inputs, output, and parameters.
			context.getConnectionManager()
					.addConnection(new WFNetReductionConnection(net, inputs, outputs, results.toString(), parameters));
		}
		// Return the output.
		return results;
	}

}
