package pl.mimuw.wfnetreduction.plugins;

import java.awt.GridLayout;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Set;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.jgraph.ProMJGraphVisualizer;
import org.processmining.models.semantics.petrinet.Marking;

import pl.mimuw.wfnetreduction.algorithms.ContractionLog;

/**
 * This plugin visualizes information about a ContractionLog.
 * 
 * @author jsroka
 * 
 */
@Plugin(name = "@0 Visualize Petri net", level = PluginLevel.PeerReviewed, returnLabels = {
		"Visualized Petrinet" }, returnTypes = {
				JComponent.class }, parameterLabels = { "Petri net", "Initial Marking" }, userAccessible = true)
@Visualizer
public class PetrinetVisualization2 {
	ContractionLog log;//the whole log
	int i = -1;//index of current possition in log

	PetrinetGraph net;//current PN
	Set<PetrinetNode> contractedNodes;//current contracted nodes
	String logString;//current log String	
	ViewSpecificAttributeMap map = new ViewSpecificAttributeMap();//map with formating to highlight contracted nodes

	JPanel topPanel = new JPanel();
	JTextArea text = new JTextArea();

	private void updateToNextStep(PluginContext context) {
		net = log.getPetrinets().get(++i);
		logString = "Use left/right arrows to change step\nStep " + i + "\n" + log.getStrings().get(i);
		contractedNodes = log.getContractedNodes().get(i);

		map = new ViewSpecificAttributeMap();
		for (PetrinetNode p : contractedNodes) {
			map.putViewSpecific(p, AttributeMap.FILLCOLOR, java.awt.Color.GRAY);
		}
		for (Place p : net.getPlaces()) {
			map.putViewSpecific(p, AttributeMap.LABEL, p.getLabel());
			map.putViewSpecific(p, AttributeMap.SHOWLABEL, true);
		}

		text.setText(logString);
		topPanel.removeAll();
		//topPanel.add(new JLabel("ALA MA KOTA"+i));
		topPanel.add(ProMJGraphVisualizer.instance().visualizeGraph(context, net, map));
		topPanel.validate();
	}

	private void updateToPreviousStep(PluginContext context) {
		net = log.getPetrinets().get(--i);
		logString = "Use left/right arrows to change step\nStep " + i + "\n" + log.getStrings().get(i);
		contractedNodes = log.getContractedNodes().get(i);

		map = new ViewSpecificAttributeMap();
		for (PetrinetNode p : contractedNodes) {
			map.putViewSpecific(p, AttributeMap.FILLCOLOR, java.awt.Color.GRAY);
		}
		for (Place p : net.getPlaces()) {
			map.putViewSpecific(p, AttributeMap.LABEL, p.getLabel());
			map.putViewSpecific(p, AttributeMap.SHOWLABEL, true);
		}

		text.setText(logString);
		topPanel.removeAll();
		//topPanel.add(new JLabel("ALA MA KOTA"+i));
		topPanel.add(ProMJGraphVisualizer.instance().visualizeGraph(context, net, map));
		topPanel.validate();
	}

	private boolean areThereNextSteps() {
		return (i < log.getPetrinets().size() - 1);
	}

	private boolean areTherePreviousSteps() {
		return (i > 0);
	}

	protected JComponent visualizeWithAll(PluginContext context, ContractionLog log, Marking m) {
		this.log = log;

		JPanel mainPanel = new JPanel();
		GridLayout mainLayout = new GridLayout(2, 1);
		mainPanel.setLayout(mainLayout);

		mainPanel.add(topPanel);
		topPanel.setLayout(new GridLayout(1, 1));
		JScrollPane scrollPane = new JScrollPane(text);
		text.setLineWrap(true);
		mainPanel.add(scrollPane);

		updateToNextStep(context);

		text.addKeyListener(new KeyListener() {

			public void keyTyped(KeyEvent e) {
			}

			public void keyReleased(KeyEvent e) {
			}

			public void keyPressed(KeyEvent e) {
				synchronized (this) {
					switch (e.getKeyCode()) {
						case 39 : //right arrow
							if (areThereNextSteps())
								updateToNextStep(context);
							break;
						case 37 : //left arrow
							if (areTherePreviousSteps())
								updateToPreviousStep(context);
							break;
					}
				}
			}
		});

		return mainPanel;

	}

	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, ContractionLog log) {
		return visualizeWithAll(context, log, null);
	}

	@PluginVariant(requiredParameterLabels = { 0, 1 })
	public JComponent visualize(PluginContext context, ContractionLog log, Marking m) {
		return visualizeWithAll(context, log, m);
	}
}